SoundBufferState

Enum SoundBufferState 

pub enum SoundBufferState {
    Generic(GenericBuffer),
    Streaming(StreamingBuffer),
}
Sound buffer is a data source for sound sources. See module documentation for more info.

Variants



Generic(GenericBuffer)

General-purpose buffer, usually contains all the data and allows random access to samples. It is also used to make streaming buffer via composition.



Streaming(StreamingBuffer)

Buffer that will be filled by small portions of data only when it is needed. Ideal for large sounds (music, ambient, etc.), because unpacked PCM data takes very large amount of RAM. Allows random access only to currently loaded block, so in general there is no true random access.

Implementations

Source

impl SoundBufferState

pub fn raw_streaming(data_source: DataSource) -> Result<Self, DataSource>

Tries to create new streaming sound buffer from a given data source. It returns raw sound buffer that has to be wrapped into Arc<Mutex<>> for use with sound sources.

pub fn raw_generic(data_source: DataSource) -> Result<Self, DataSource>

Tries to create new generic sound buffer from a given data source. It returns raw sound buffer that has to be wrapped into Arc<Mutex<>> for use with sound sources.

Methods from Deref<Target = GenericBuffer>

pub fn external_data_path(&self) -> &Path

In case if buffer was created from file, this method returns file name. Can be useful for serialization needs where you just need to know which file needs to be reloaded from disk when you loading a saved game.

pub fn set_external_data_path(&mut self, path: PathBuf) -> PathBuf

Sets new path for external data source.

pub fn is_empty(&self) -> bool

Checks if buffer is empty or not.

pub fn samples(&self) -> &[f32]

Returns shared reference to an array with samples.

pub fn samples_mut(&mut self) -> &mut [f32]

Returns mutable reference to an array with samples that could be modified.

pub fn channel_count(&self) -> usize

Returns exact amount of channels in the buffer.

pub fn sample_rate(&self) -> usize

Returns sample rate of the buffer.

pub fn duration(&self) -> Duration

Returns exact duration of the buffer.