GenericBuffer

Struct GenericBuffer 

pub struct GenericBuffer { /* private fields */ }
Generic sound buffer that contains decoded samples and allows random access.

Implementations

Source

impl GenericBuffer

pub fn new(source: DataSource) -> Result<Self, DataSource>

Creates new generic buffer from specified data source. May fail if data source has unsupported format, corrupted, etc.

Notes

DataSource::RawStreaming is not supported with generic buffers, use streaming buffer instead!

Data source with raw samples must have sample count multiple of channel count, otherwise this function will return Err.

pub fn external_data_path(&self) -> &Path

In case if buffer was created from file, this method returns file name. Can be useful for serialization needs where you just need to know which file needs to be reloaded from disk when you loading a saved game.

pub fn set_external_data_path(&mut self, path: PathBuf) -> PathBuf

Sets new path for external data source.

pub fn is_empty(&self) -> bool

Checks if buffer is empty or not.

pub fn samples(&self) -> &[f32]

Returns shared reference to an array with samples.

pub fn samples_mut(&mut self) -> &mut [f32]

Returns mutable reference to an array with samples that could be modified.

pub fn channel_count(&self) -> usize

Returns exact amount of channels in the buffer.

pub fn sample_rate(&self) -> usize

Returns sample rate of the buffer.

pub fn duration(&self) -> Duration

Returns exact duration of the buffer.