StreamingBuffer

Struct StreamingBuffer 

pub struct StreamingBuffer { /* private fields */ }
Streaming buffer for long sounds. Does not support random access.

Implementations

Source

impl StreamingBuffer

pub const STREAM_SAMPLE_COUNT: usize = 44100

Defines amount of samples per channel which each streaming buffer will use for internal buffer.

pub fn new(source: DataSource) -> Result<Self, DataSource>

Creates new streaming buffer using given data source. May fail if data source has unsupported format or it has corrupted data. Length of internal generic buffer cannot be changed but can be fetched from StreamingBuffer::STREAM_SAMPLE_COUNT

Notes

This function will return Err if data source is Raw. It makes no sense to stream raw data which is already loaded into memory. Use Generic source instead!

pub fn duration(&self) -> Option<Duration>

Returns total duration of data. Can be None if internal decoder does not supports seeking.

Methods from Deref<Target = GenericBuffer>

pub fn external_data_path(&self) -> &Path

In case if buffer was created from file, this method returns file name. Can be useful for serialization needs where you just need to know which file needs to be reloaded from disk when you loading a saved game.

pub fn set_external_data_path(&mut self, path: PathBuf) -> PathBuf

Sets new path for external data source.

pub fn is_empty(&self) -> bool

Checks if buffer is empty or not.

pub fn samples(&self) -> &[f32]

Returns shared reference to an array with samples.

pub fn samples_mut(&mut self) -> &mut [f32]

Returns mutable reference to an array with samples that could be modified.

pub fn channel_count(&self) -> usize

Returns exact amount of channels in the buffer.

pub fn sample_rate(&self) -> usize

Returns sample rate of the buffer.

pub fn duration(&self) -> Duration

Returns exact duration of the buffer.