pub struct InputFilter { /* private fields */ }Expand description
Input filter is used to transform samples in desired manner, it is based on generic second order biquad filter. See docs for Biquad filter.
Implementations