pub enum SoundBufferState {
Generic(GenericBuffer),
Streaming(StreamingBuffer),
}Expand description
Sound buffer is a data source for sound sources. See module documentation for more info.
Variants
Generic(GenericBuffer)
General-purpose buffer, usually contains all the data and allows random access to samples. It is also used to make streaming buffer via composition.
Streaming(StreamingBuffer)
Buffer that will be filled by small portions of data only when it is needed. Ideal for large sounds (music, ambient, etc.), because unpacked PCM data takes very large amount of RAM. Allows random access only to currently loaded block, so in general there is no true random access.
Implementations
Sourceimpl SoundBufferState
impl SoundBufferState
Sourcepub fn raw_streaming(data_source: DataSource) -> Result<Self, DataSource>
pub fn raw_streaming(data_source: DataSource) -> Result<Self, DataSource>
Tries to create new streaming sound buffer from a given data source. It returns raw sound buffer that has to be wrapped into Arc<Mutex<>> for use with sound sources.
Sourcepub fn raw_generic(data_source: DataSource) -> Result<Self, DataSource>
pub fn raw_generic(data_source: DataSource) -> Result<Self, DataSource>
Tries to create new generic sound buffer from a given data source. It returns raw sound buffer that has to be wrapped into Arc<Mutex<>> for use with sound sources.
Methods from Deref<Target = GenericBuffer>
Sourcepub fn external_data_path(&self) -> &Path
pub fn external_data_path(&self) -> &Path
In case if buffer was created from file, this method returns file name. Can be useful for serialization needs where you just need to know which file needs to be reloaded from disk when you loading a saved game.
Sourcepub fn set_external_data_path(&mut self, path: PathBuf) -> PathBuf
pub fn set_external_data_path(&mut self, path: PathBuf) -> PathBuf
Sets new path for external data source.
Sourcepub fn samples_mut(&mut self) -> &mut [f32]
pub fn samples_mut(&mut self) -> &mut [f32]
Returns mutable reference to an array with samples that could be modified.
Sourcepub fn channel_count(&self) -> usize
pub fn channel_count(&self) -> usize
Returns exact amount of channels in the buffer.
Sourcepub fn sample_rate(&self) -> usize
pub fn sample_rate(&self) -> usize
Returns sample rate of the buffer.